****** Make Friends with SAS;
****** Created by Lesa Hoffman;

*******************************************************************************
*******             SAS SYNTAX AND DATA NAME RULES            *******
******************************************************************************;

***** WRITE COMMENTS TO HELP YOU! *****;
/* You can comment like this for large chunks of code */
* Or you can comment line by line as needed like this;
* Try not to use unmatched quotes in comments, or SAS gets confused;

/*  USE COLORS TO HELP YOU!
    Comments are GREEN 
    SAS system commands are BLUE
    Variables and file names are BLACK
    Entered data should be shaded YELLOW
    If you see RED, something is wrong 
    Libraries and Labels are PURPLE
        -- anything in PURPLE is case-sensitive and space-sensitive
    Otherwise SAS is NOT case-sensitive or space-sensitive  */
   
* Each SAS command (that starts with a BLUE word) must end with a semi-colon;
* Data transformations (e.g., making new variables) must end with a semi-colon;

* SAS dataset names have two parts, separated by a period: Library.Filename
    Library = where the file is located on your computer
              the default place is "work", which is a temporary directory
              "work" is the safest place to store data files when you are 
              transforming them because it wont ruin anything permanent
    Filename = name of file (without extension)
You can have as many different libraries (places) and files as you want;


************************************************
***   IMPORTING AND EXPORTING SAS DATASETS   ***
************************************************;

* Step 1: Tell SAS where your data files are by making a library;
* Step 2: Tell SAS the old (SET) and new (DATA) names of the datafile;

* A LIBRARY is name for a specific PLACE (not a file) on your computer
  LIBRARY names must be 8 characters or less, with no special characters
  Once you have defined a library using a LIBNAME staement, you can access 
  existing SAS data sets or save newly created data sets to a permanent file;

* Step 1: I have named my library "filepath";
LIBNAME filepath "C:\Dropbox\17_CLDP944\CLDP944_SAS";

* Step 1, if SAS university edition and I have a shortcut for Dropbox;
LIBNAME filepath "/folders/myshortcuts/Dropbox/17_CLDP944/CLDP944_SAS";



* Your turn -- make a library with your name (up to 8 characters);



* It is a REALLY GOOD IDEA to immediately copy whatever SAS data file you
  will be working with to the "work" library, which is a temporary location;

* DATA creates a new SAS data set in the work library called "newsasfile"
  FROM (SET) the permanent SAS file "oldsasfile" in the "filepath" location;
DATA work.newsasfile;
    SET filepath.oldsasfile;
RUN;

* This code is equivalent -- if you dont specify a library, it goes to "work";
* Except in SAS university edition, where you need to say work.file explicitly;
DATA newsasfile;
    SET filepath.oldsasfile;
RUN;

* Your turn -- import the SAS dataset "saspractice";



* To save a SAS file from temporary "work" back to permanent library:
  DATA creates a SAS data set in the permanent library called "sasfilesaved"
  FROM (SET) the existing SAS file "sasfile" in the "work" location;
DATA filepath.sasfilesaved;
    SET work.sasfile;
RUN;

* Your turn -- save the SAS dataset "saspractice" as "SASfinal";




*************************************************
***   IMPORTING AND EXPORTING SPSS DATASETS   ***
************************************************;

* IMPORT SPSS data using exact, hard-coded path, filename, and extension;
* New SAS file is in "work" library and is called "spssfile";
* DBMS ("database management system") tells SAS what kind of file it is
* "REPLACE" means write over it if it already exists (is ok in "work");
PROC IMPORT DATAFILE="C:\Dropbox\17_CLDP944\CLDP944_SAS\spssfile.sav" 
    OUT=work.sasfile DBMS=SAV REPLACE;
RUN;
* If using SAS university edition instead;
PROC IMPORT DATAFILE="/folders/myshortcuts/Dropbox/17_CLDP944/CLDP944_SAS/spssfile.sav" 
    OUT=work.sasfile DBMS=SAV REPLACE;
RUN;

* Your turn -- import "spsspractice.sav";




* SAS does NOT have value labels for variables like SPSS does
  SAS has "formats", which are labels that can be used in data and/or output
  Variables with SPSS value labels are read in as formats by default
  This means your original variable values are not visible, and 
  categorical variables are now sorted by their labels, not their values;

* To fix it, save it as a new file (same SET, new DATA)
  and add a FORMAT statement that removed all formats from data
  The keyword _ALL_ refers to all variables, or you can list specific variables;
DATA work.spssfile_nolabels; 
    SET work.spssfile; 
    FORMAT _ALL_; 
RUN;

* Your turn -- take the labels out of the spsspractice file;




* To save formats to a separate SAS dataset, use CNTLOUT and name new datafile;
PROC FORMAT CNTLOUT=work.DataFormats LIBRARY=work.formats; 
RUN;

* By default SAS shows variable labels instead of names -- here is how to fix it

1) Select the SAS Explorer window to make it the active window.
2) Select Tools->Options->Explorer... from the pull-down menu.
3) Select the "Members" tab on the Explorer Options window.
4) Select "Table" and then hit "Edit".
5) Select "&Open" and then hit "Edit"
6) Add "colheading=name" (no quotes) to the action command.
7) Repeat steps 4-6 for member type &View Columns.
8) OK your way out of the Explorer Options window.


* EXPORT to SPSS data using exact, hard-coded path, filename, and extension
  DATA says what SAS file to export, OUTFILE says where file goes, DBMS says what kind;
PROC EXPORT DATA=work.sasfile OUTFILE="C:\Dropbox\17_CLDP944\CLDP944_SAS\savedfile.sav" 
    DBMS=SAV REPLACE; 
RUN;

* Your turn -- export "saspractice" to SPSS and call it "spssversion";



* Importing any kind of non-SAS data requires an exact file path 
  To make it easier on ourselves, we can use a macro variable "filepath";

* A MACRO variable is a programming variable that works like "find & replace"
  Here, "filepath" functions like a library and abbreviates the path
  Then, if you change your folder you only need to change it ONCE below;

* %LET statement defines a macro variable for the file path we wrote out before;
%LET filepath=C:\Dropbox\17_CLDP944\CLDP944_SAS;

* Now we can use this abbreviation by adding "&" in front and "." in back;  
PROC EXPORT DATA=work.sasfile OUTFILE="&filepath.\savedfile.sav" 
    DBMS=SAV REPLACE; 
RUN;

* Your turn -- export "saspractice" to SPSS again, using the &filesave. macro variable instead;



*************************************************
***  IMPORTING AND EXPORTING STATA DATASETS   ***
************************************************;

* IMPORT stata data using exact, hard-coded path, filename, and extension;
* New SAS file is in "work" library and is called "statafile";
* DBMS ("database management system") tells SAS what kind of file it is
* "REPLACE" means write over it if it already exists (is ok in "work");
PROC IMPORT DATAFILE="C:\Dropbox\17_CLDP944\CLDP944_SAS\statafile.dta" 
    OUT=work.sasfile DBMS=DTA REPLACE;
RUN;
* Using our new &filesave. abbrevation to do the same thing;
PROC IMPORT DATAFILE="&filesave.\statafile.dta" 
    OUT=work.sasfile DBMS=DTA REPLACE;
RUN;

* Your turn -- import "statapractice.dta";


* EXPORT to STATA data using exact, hard-coded path, filename, and extension
  DATA says what SAS file to export, OUTFILE says where file goes, DBMS says what kind;
PROC EXPORT DATA=work.sasfile OUTFILE="C:\Dropbox\17_CLDP944\CLDP944_SAS\savedfile.dta" 
    DBMS=DTA REPLACE; 
RUN;
* Using our new &filesave. abbrevation to do the same thing;
PROC EXPORT DATA=work.sasfile OUTFILE="&filesave.\savedfile.dta" 
    DBMS=DTA REPLACE;
RUN;

**************************************************
***  IMPORTING AND EXPORTING EXCEL WORKSHEETS  ***
*************************************************;

* IMPORT: New SAS data set in the work library called "excelfile"
  from the Excel workbook in DATAFILE= location from SHEET= 
  "GETNAMES" reads in the first row as variable names;
    
* Using abbreviated path we just made to import OLD version of excel files xls;
PROC IMPORT DATAFILE="&filepath.\excelfile.xls" OUT=work.excelfile DBMS=EXCEL REPLACE; 
    SHEET="sheetname"; 
    GETNAMES=YES; 
RUN;

* Using abbreviated path we just made to import NEW version of excel files xlsx;
PROC IMPORT DATAFILE="&filepath.\excelfile.xlsx" OUT=work.excelfilex DBMS=XLSX REPLACE; 
    SHEET="sheetname"; 
    GETNAMES=YES; 
RUN;

* Your turn -- import "excelpractice.xlsx" with sheetname "mydata";



* EXPORT: DATA lists file to be exported, OUTFILE lists name and location of new file;
* Using abbreviated path we just made;
PROC EXPORT DATA=work.sasfile OUTFILE="&filepath.\savedfile.xls" DBMS=EXCEL REPLACE; 
RUN;
PROC EXPORT DATA=work.sasfile OUTFILE="&filepath.\savedfile.xlsx" DBMS=XLSX REPLACE; 
RUN;

* Your turn -- export "saspractice" to XLS and call it "excelversion";




****************************************
***  IMPORTING/EXPORTING CSV FILES   ***
****************************************;

* Certain versions of SAS do not permit the IMPORT/EXPORT procedures to be used
  with excel. In that case, it is best to save the excel file as a .csv file,
  a comma-delimited file, and import/export it in that format instead;

***** TO IMPORT A CSV FILE;
PROC IMPORT DATAFILE="&filepath.\csvfile.csv" OUT=work.sasfile DBMS=CSV REPLACE;  
    DATAROW=2;      * Which row to start importing from;
    GETNAMES=YES;   * Select NO if there are no names on the top row;
RUN;

***** TO EXPORT TO A CSV FILE (useful for Mplus);
PROC EXPORT DATA=work.sasfile OUTFILE="&filepath.\savedfile.csv" DBMS=CSV REPLACE; 
RUN;



*******************************************************************************
*******             TRANSFORMATIONS TO SAS DATASETS           *******
******************************************************************************;

**********************
***  SORTING DATA  ***
*********************;

* Sort BY variables, ascending by default (add DESCENDING before variable otherwise);
PROC SORT DATA=work.datafilename; 
    BY IDvariable timevariable;  * List as many variables as you want, in order;
RUN;

* Your turn -- sort saspractice by ID;



* PROC SORT can be used to eliminate duplicates: 
  NODUPKEY = all observations with the same BY values will be deleted.
  Sorted, non-duplicate dataset is saved via OUT as "sorteddatafilename" 
  Duplicate cases get saved via DUPOUT into "duplicates" instead;
PROC SORT DATA=work.datafilename NODUPKEY OUT=work.sorteddatafilename DUPOUT=work.duplicates; 
    BY var1 var2;  * List variables to select for duplicates on or use _ALL_;
RUN;


*****************************
***   MERGING data files  ***
****************************;

* DATA step names file to be created from merged datasets.
  MERGE lists datasets to be combined when ADDING CASES.
  If variables differ, those cases without will get missing values.
  Here, observations will be matched by IDvariable and timevariable;

* Sort each data set the same way, then merge them together;
* Note that the same variables are used to sort and merge;
PROC SORT DATA=work.data1; 
    BY IDvariable timevariable; 
RUN;
PROC SORT DATA=work.data2; 
    BY IDvariable timevariable; 
RUN;
DATA work.databoth; 
    MERGE work.data1 work.data2; 
    BY IDvariable timevariable; 
RUN; 


******************************************
***  KEEP, DROP, or REORDER Variables  ***
*****************************************;

* The following commands can be used in DATA steps:
    KEEP    keeps listed variables only
    DROP    drops listed variables only
    RETAIN  used to re-order existing variables as listed in file
            must be used immediately after DATA command
    By default, all variables are kept;

* Here, we make a new dataset that KEEPS only 3 variables;
DATA work.shorterdatafilename; 
    SET work.datafilename; 
    KEEP ID var1 var2;
RUN;

* Here, we make a new dataset that DROPS only 3 variables;
DATA work.shorterdatafilename; 
    SET work.datafilename; 
    DROP ID var1 var2;
RUN;

* We can use RETAIN to re-order variables in dataset;
* Any variables not listed on RETAIN get same place in dataset;
* RETAIN must be used IMMEDIATELY after the DATA command;
* Here, we want "firstvar" first and "secondvar" to be second;
* All other variables get put after it in original order;
DATA work.newdatafilename;
    RETAIN firstvar secondvar;
    SET work.datafilename;
RUN;


****************************
***  KEEP or DROP Cases  ***
***************************;

* WHERE statement is like FILTER in SPSS;
* WHERE can appear after any DATA or PROC statement 
  Select cases using conditional operators: EQ, NE, GT, LT, 
  GE, LE, AND, OR, IS NOT MISSING, or other functions like NMISS;

* This creates a new dataset with only cases where Sex=1;
DATA work.girlsonly; 
    SET work.olddata;
    WHERE Sex=1;
RUN;

* This creates a new dataset with only cases where Sex=1 and group=1;
DATA work.girlsgroup1only; 
    SET work.olddata;
    WHERE Sex=1 AND group=1;
RUN;

* This creates a new dataset with only complete cases for var1 and var2; 
DATA work.completecases; 
    SET work.olddata;
    WHERE NMISS(var1, var2)=0; * Where number missing for (variables) is 0;
RUN;


*******************************************************************************
*******              WAYS OF DESCRIBING/SUMMARIZING DATA          *******
******************************************************************************;

*******************************
***    SAVING SAS OUTPUT    ***
******************************;

* Output goes to the results window by default, but you can save it to a file using ODS;

* %LET statement defines a macro variable for the file path we wrote out before;
%LET filepath=C:\Dropbox\17_CLDP944\CLDP944_SAS;

* To save your output in to something you can open in excel, use html or xls extension;
ODS HTML FILE="&filepath.\Output.xls" STYLE=MINIMAL;
    * PROC commands go here;
ODS HTML CLOSE; * Shuts HTML file so you can open it;
ODS HTML;       * Re-opens HTML output window in SAS;

* To save your output in to something you can open in Word:
    STARTPAGE controls whether each table starts on a new page
    BODYTITLE prints titles inside text, rather than as a page header
    STYLE offers format options (HTMLBLUE is the SAS output default)
     (I also like statdoc and fancyprinter for rtf, minimal for excel);

ODS RTF FILE="&filepath.\Output.rtf" STARTPAGE=NO BODYTITLE STYLE=FANCYPRINTER;
    * PROC commands go here;
ODS RTF CLOSE; * Shuts RTF file so you can open it;


*******************************
***  CREATING VALUE LABELS  ***
******************************;

* Use PROC FORMAT instead to create VALUE labels to use in data or analyses later;

* Here, we create two new formats (value labels) called "FGender" and "Fgroup";
* Note that I keep the value so that the order stays the same;
PROC FORMAT LIBRARY=work;
    VALUE FGender 0="0.Male" 1="1.Female";
    VALUE FGroup  0="0.Control" 1="1.Treatment";
    VALUE Fcatvar 0="0.No" 1="1.Yes";
RUN;
* Now we add the new format to a variable by adding "." after it in a FORMAT statement;
* These labels will appear in all output for this variable;
* You can also use FORMAT statements just to label output from an analysis;
DATA work.datafile;
    SET work.datafile;
    FORMAT female Fgender. group Fgroup.;
RUN;


********************************
***  DESCRIPTIVE STATISTICS  ***
*******************************;

* To print frequency tables, where TABLE = variables to use;
TITLE1 "Descriptive Statistics for My Categorical Variables";
PROC FREQ DATA=work.datafile;               
    TABLE catvar1 catvar2 catvar1*catvar2;  * Use of "*" gives crosstab;
    FORMAT catvar1 catvar2 Fcatvar.;        * Both variables will use format "Fcatvar" defined above;
RUN; TITLE1;

* To print frequency tables by group -- must sort by group first;
PROC SORT DATA=work.datafile; BY group; RUN;
TITLE1 "Descriptive Statistics for My Categorical Variables by Group";
PROC FREQ DATA=work.datafile;   
     BY group;  
    TABLE catvar1 catvar2 catvar1*catvar2;  * Use of "*" gives crosstab;
    FORMAT catvar1 catvar2 Fcatvar.;        * Both variables will use format "Fcatvar" defined above;
RUN; TITLE1;

* To print sample descriptive statistics for continuous variables;
* Optional arguments: N MEAN STDDEV MIN MAX SKEW KURTOSIS;
* NDEC= controls number of decimal places;
TITLE1 "Descriptive Statistics for My Continuous Variables For Group=1";
PROC MEANS NDEC=2 DATA=work.datafilename;
    VAR var1 var2 var4-var10;   * Variables to use;
    WHERE group=1;              * Select only these cases, omit to use all cases;
RUN; TITLE1;

* To print sample descriptive statistics for continuous variables by group;
* If you use CLASS instead of BY, you have more options and dont have to sort first;
TITLE1 "Descriptive Statistics for My Continuous Variables by Group and Time";
PROC MEANS NDEC=2 DATA=work.datafilename;
    CLASS group time;           * Will print per level of CLASS variables;
    WAYS 0 1 2;                 * 0=overall, 1=marginal, 2=by unique combo;
    VAR var1 var2 var4-var10;   * Variables to use;
RUN; TITLE1;

* PROC UNIVARIATE does all of the above and a lot more;
TITLE1 "Descriptive Statistics and Plots";
PROC UNIVARIATE DATA=work.datafile FREQ;    * FREQ = print frequencies;
    CLASS group;                            * Will print per level of CLASS variables;
    VAR var1;                               * Variables to provide summary info for;
    * Distribution plots, with estimated shapes requested after;
      HISTOGRAM var1 / NORMAL LOGNORMAL;            * Overlay these curves and run tests;
        INSET NORMAL LOGNORMAL / FORMAT=6.2 POS=NE; * Put summary stats in a box in northeast corner of plot;
      QQPLOT var1 / NORMAL(MU=est SIGMA=est);       * Quantile-Quantile Plot assuming normal;
        INSET NORMAL / FORMAT=6.2;                  * Put summary stats in a box in the plot (default is northwest);
      PROBPLOT var1 / NORMAL(MU=est SIGMA=est);     * Probability Plot assuming normal;
        INSET NORMAL / FORMAT=6.2;                  * Put summary stats in a box in the plot;
      QQPLOT var1 / LOGNORMAL(THRESHOLD=est SCALE=est SHAPE=est); * Quantile-Quantile Plot assuming lognormal;
        INSET  LOGNORMAL / FORMAT=6.2;                            * Put summary stats in a box in the plot;
      PROBPLOT var1 / LOGNORMAL(THRESHOLD=est SCALE=est SHAPE=est);  * Probability Plot assuming lognormal;
        INSET  LOGNORMAL / FORMAT=6.2;                               * Put summary stats in a box in the plot;
RUN; QUIT; * Need both RUN and QUIT; TITLE1;

* To print correlations (and/or covariance matrices -- add COV to PROC CORR line);    
* OUT saves correlations to a SAS dataset;
TITLE1 "Correlations for My Continuous Variables For Group=1";
PROC CORR DATA=work.datafilename OUT=work.corrs; 
    VAR var1 var2 var4-var10;   * Variables to use;
    WHERE group=1;              * Select only these cases, omit to use all cases;
RUN; TITLE1;

* You can export the correlation matrix to excel for easy table-making;
PROC EXPORT DATA=work.corrs OUTFILE="&filepath.\Results.xls" DBMS=EXCEL REPLACE; 
    SHEET="My Correlations";
RUN;

* Your turn -- run descriptives and correlations for gps1--gps5 in spsspractice;
* Save all your output to an excel file;





*******************************************************************************
*******         TRANSFORMATIONS TO VARIABLES WITHIN DATASETS          *******
******************************************************************************;
        

****************** Critical Point ******************************;
* Transformations must happen within a DATA step: use DATA + SET;
* If you forget the SET, you will create an empty file!;
****************************************************************;

* These transformations can be used if all values necessary are in the same row
  If summarizing across rows, you'll need PROC MEANS BY ID instead (see day 1);

* IF-THEN allows for conditional processing
  Make sure to put " " around string variable characters
  Use parentheses to make sure expression is read as intended
  "IF" gets paired with other relationsal operators such as:
    "equal to"              =   EQ
    "not equal to"              ~=  NE  
    "less than"             <   LT
    "less than or equal to"     <=  LE
    "greater than"              >   GT
    "greater than or equal to"  >=  GE
    "and"                   &   AND
    "or"                        |   OR
    "not"                   ~   NOT ;

* Can abbreviate lists of variables for KEEP and DROP two ways:
        "-"  refers to variables numerically
        "--" refers to variables positionally   
    var1-var50 refers to var1, var2, var3...var50
    var1--var50 refers to var1, (next var in file)... var50;

* Transformations must happen within a DATA step: use DATA + SET
  If you forget the SET, you will create an empty file!;
 DATA work.datafile; SET work.datafile;
      * Transformations go here ....;
 RUN;

********************************
***   MAKING NEW VARIABLES   ***
*******************************;

* How to rename and relabel existing variables;
    RENAME oldvar1=newvar1 oldvar2=newvar2;
* Labels still refer to OLD variable name because RENAME is processed last!!!;
    LABEL oldvar1= "newvar1: Label of New Variable1"
          oldvar2= "newvar2: Label of New Variable2";

 * Example of making a new variable: centering a continuous predictor at 30;
    c_pred = pred - 30;
    LABEL c_pred= "c_pred: Predictor Centered at 30 (0=30)";

* Example of making a new variable by summarizing existing variables
  Here, we are saving a new variable that is the mean of existing variables
  These lines illustrate the different ways to refer to a series of variables
  Dont forget the "OF" or it will try to subtract them when using -;
    meanvar = MEAN(OF var1, var2, var3, var4);  * long way;
    meanvar = MEAN(OF var1-var4);               * contiguous names;
    meanvar = MEAN(OF var1--var4);          * contiguous positions;
    LABEL meanvar= "meanvar: Mean of Var1-Var4";

* Note this is NOT equivalent to the code above
  When using operators, if ANY are missing, the meavar will be missing;
    meanvar = (var1 + var2 + var3 + var4) / 4;
    LABEL meanvar= "meanvar: Mean of Var1-Var4 (only if all 4)";

* NMISS function returns number MISSING rsponses for numeric variables 
  N function returns number of VALID responses for numeric variables;
    nmiss10  = NMISS(OF var1--var10);
    nvalid10 = N(OF var1--var10);
    LABEL nmiss10= "nmiss10: Number Missing for Var1-10"
          nval10=  "nval10: Number Answered for Var1-10";

* There is no direct way to specify minimum number to be computed;
* NMISS function can count number of missing values and IF to select;
    IF NMISS(OF var1-var4) LE 1 THEN DO; 
        meanvar = MEAN(OF var1-var4); maxvar = MAX(OF var1-var4); END;
    ELSE DO; 
        meanvar = .; maxvar = .; END;
* Otherwise, N function can count number of valid values and IF to select;
    IF N(OF var1-var4) GE 3 THEN DO; 
        meanvar = MEAN(OF var1-var4); maxvar = MAX(OF var1-var4); END;
    ELSE DO; 
        meanvar = .; maxvar = .; END;
    LABEL meanvar= "meanvar: Mean of Var1-Var4 (if at least 3)"
          maxvar=  "maxvar:  Max  of Var1-Var4 (if at least 3)";

* Hundreds of other possible functions -- ask the google for help;
* Functions can include variables or numeric constants as needed;
* Here are some more types of transformation;
    newvar = oldvar**2;             * square old variable to make new;
    logvar = LOG(var);              * taking natural log;
    log10var = LOG10(var);          * taking log-base 10;
    maxvar = MAX(OF var1-var10);    * maximum value across variables;
    minvar = MIN(OF var1-var10);    * minimum value across variables;
    sumvar = SUM(OF var1-var10);    * sum across variables;
    stdvar = STD(OF var1-var10);    * standard deviation across variables;

* Example of saving the row number as permanent variable using _N_ keyword;
    IDvar = _N_;
    LABEL IDvar "IDvar: New ID variable";

***********************************************************************************;
* There is no RECODE in SAS - you have to use IF-THEN,ELSE yourself
  You can compute new variables, but you don't need the word COMPUTE;

* IF-THEN,ELSE-THEN works for single transformations
* Need IF-THEN-DO-END,ELSE-DO-END for multiple transformations;

* The IN function searches a variable for a series of possible values;
* This gives the variable 'pick2468' a value of 1 if var1 = 2 or 4 or 6 or 8;
    IF var1 IN(2,4,6,8) THEN pick2468=1; ELSE pick2468=0;
    LABEL pick2468= "pick2468: Var1=2,4,6,or8 (0=no, 1=yes)";

* Example of creating categories from a continuous variable;
* missing or <1 = missing, 1 to 2 = 1, 2 to highest = 2;
* The "." = system missing, 999 for unanticipated value;
* Specify missing values first, because in SAS . is < 0;
         IF var1 = .                 THEN catvar1 = .; 
    ELSE IF var1 LT 1                THEN catvar1 = .; 
    ELSE IF var1 GE 1 AND var1 LT 2  THEN catvar1 = 1; 
    ELSE IF GE 2                     THEN catvar1 = 2; 
    LABEL catvar1= "catvar1: Var1 = missing, 1, or 2";

* It may be easier to use the ROUND function for continuous variables;
    roundyear = ROUND(year,2);
    LABEL roundyear= "roundyear: Year Rounded to Nearest 2 Years";
    decade = ROUND(year,10);
    LABEL decade= "decade: Year Rounded to Nearest Decade";

* Example of reverse coding -- specify missing values first;
         IF var1=. THEN rvar1=.; 
    ELSE IF var1=5 THEN rvar1=1;
    ELSE IF var1=4 THEN rvar1=2; 
    ELSE IF var1=3 THEN rvar1=3; 
    ELSE IF var1=2 THEN rvar1=4; 
    ELSE IF var1=1 THEN rvar1=5;
    ELSE                rvar1=.; 
    LABEL rvar1= "rvar1: Var1 Reverse-Coded";

* To do this for more than one variable at a time, use an ARRAY instead
  ARRAY is a short-hand for a list of variables
  ARRAY names must not duplicate any variable names
  Parentheses after array name is #variables, followed by variable list:
  Arraynam(1)=var1, arraynam(2)=var2, etc;

* Here is an example that reverse codes 100 variables;
    ARRAY anew (100) newvar1 -  newvar100;  * Ordered by name;
    ARRAY aold (100) oldvar1 -- oldvar100;  * Ordered in dataset;
    DO i=1 TO 100 BY 1; * DO loop says do what follows 100 times;
             IF aold(i)=. THEN anew(i)=.; 
        ELSE IF aold(i)=5 THEN anew(i)=1;
        ELSE IF aold(i)=4 THEN anew(i)=2; 
        ELSE IF aold(i)=3 THEN anew(i)=3; 
        ELSE IF aold(i)=2 THEN anew(i)=4; 
        ELSE IF aold(i)=1 THEN anew(i)=5;
        ELSE                   anew(i)=.; 
    END;     * END is needed to shut off DO;
    DROP i;  * Counting variable is removed from data;

* Now make SAS write all 100 labels with a macro program;
%MACRO MakeMyLabels;
    %DO num=1 %TO 100;
        LABEL newvar&num.= "Variable &num. recoded";
    %END;
%MEND MakeMyLabels;
* This part executes the above program;
%MakeMyLabels;


***************************************************
***  Aggregating/summarizing data across rows   ***
**************************************************;

* PROC MEANS can be used to create summary data across observations.
  BY gives you a summary per unique value.
  NOPRINT suppresses output -- it is unnecessary here. 
  VAR lists variables to be summarized.
  OUTPUT OUT= lists datafile new summary variables get saved to.
  Many available functions: MEAN, STD, SUM, MIN, MAX, N... 
  Format: FUNCTION(variable names) = new variable names;

* Data MUST be sorted with the BY variable first;
PROC SORT DATA=work.datafilename; BY ID; RUN;
PROC MEANS NOPRINT DATA=work.datafilename;  * NOPRINT suppresses output;
    BY ID;                  * Summary per value of ID;
    VAR var1 var2;          * Do it for these variables;    
* Function(old vars) = new vars;
    OUTPUT OUT=work.summaryfile
        MEAN(var1 var2) = var1mn var2mn     
        STD (var1 var2) = var1sd var2sd
        N   (var1 var2) = var1n  var2n;  * A total of 6 new variables;  
RUN;

* Merging summary file back into original data;
DATA work.datafilename;
    MERGE work.datafilename work.summaryfile;
    BY ID;
* Drop unnecessary variables created by PROC MEANS;
    DROP _TYPE_ _FREQ_;
RUN;

    
*****************************************************************
***  Go from MULTIVARIATE (wide) data to STACKED (long) data  ***
****************************************************************;

* Creates stacked variables wave, IV, and DV for each time point. 
  All other variables are copied down the rows by default.
  Must include all variables per time point on same line before OUTPUT. 
  OUTPUT then writes the new line of data.
  Note: all old multivariate variables get included still. 
  If a variable is missing, you must explicitly include the "."
  See example using ARRAYS to do this more efficiently;

* First import existing multivariate data;
PROC IMPORT DATAFILE="&filepath.\multivprac.sav" 
    OUT=work.multivprac DBMS=SAV REPLACE;
RUN;
* Now created "stacked" version of data;
DATA work.stackprac;        * New file;
    SET work.multivprac;    * Old file;
        wave=1; DV=DVTime1; age=AgeTime1; OUTPUT;   * One OUTPUT row per occasion; 
        wave=2; DV=DVTime2; age=AgeTime2; OUTPUT;   
        wave=3; DV=DVTime3; age=AgeTime3; OUTPUT;       
        wave=4; DV=DVTime4; age=AgeTime4; OUTPUT; 
    LABEL wave=  "wave: occasion of measurement"
          DV=   "DV: New Dependent Variable over time"
          age=  "age: New Age over time";
* The original multivariate variables will still be in the file
  Deleting them is unnecessary, but if you wanted to, here is how;
    * Demonstrating numeric abbreviation -;
         DROP DVTime1-DVTime4 AgeTime1-AgeTime4;
    * Demonstrating positional abbreviation --;
        * DROP DVTime1--AgeTime4;
RUN; 

*  ARRAY is a short-hand for a list of variables
    ARRAY names must not duplicate any variable names
    Parentheses after array name is #variables, followed by variable list -
    Arraynam(1)=var1, arraynam(2)=var2, etc;  

* Here is an example that uses ARRAYs to do multivariate --> stacked data
  Use a separate ARRAY statement for each variable group
  Change default counting to 13-17 to be consistent with activity data;
DATA work.stackprac;        * New file;
    SET work.multivprac;    * Old file;
    ARRAY aDV  (4) DVTime1-DVTime4;
    ARRAY aAge (4) AgeTime1-AgeTime4;
    * Write 4 lines of data for waves 1 to 4;
    DO wave=1 TO 4 BY 1;
        DV=aDV(wave); Age=aAge(wave); OUTPUT;
    END;
RUN;

* Your turn -- import SAS dataset SAS_Chapter7b into the work library, 
  then stack to make 4 new variables: occasion, age, risky, monitor;











*****************************************************************
***  Go from STACKED (long) data to MULTIVARIATE (wide) data  ***
****************************************************************;

* You can use PROC TRANSPOSE to make only ONE variable multivariate at a time.
  See example at end using ARRAYS to do this for more than one variable;

* First specify OLD (DATA=) and NEW (OUT=) dataset names
  PREFIX will add that text to beginning of new transposed variables.
  Transpose BY each level - aggregates into one row per unique value.
  List all constant variables on BY statement, too.
  ID - Variable that names the new columns.
  VAR - Variables to be transposed;

PROC TRANSPOSE DATA=work.stacked OUT=work.mult PREFIX=Wave_;        
    BY ID gender;   * Transpose per unique value of BY;                                                 
    ID wave;        * Variable to define new columns;                                                                   
    VAR DV;     * Variable in old rows;                 
RUN;

* Here is an example that uses ARRAYs to do stacked --> multivariate data
  Make sure data are sorted by idnum and time first;
PROC SORT DATA=work.stackprac; BY ID wave; RUN;
DATA work.multivprac2; SET work.stackprac; BY ID;
    * Tell it to keep ID and new multivariate variables (and whichever others);
    KEEP ID Gender DVTime1-DVTime4 AgeTime1-AgeTime4;
    * RETAIN here is used to hang onto values across rows;
    RETAIN DVTime1-DVTime4 AgeTime1-AgeTime4;
    * Separate ARRAY for each group of variables to be restructured;
    ARRAY aDV (4) DVTime1-DVTime4;
    ARRAY aAge(4) AgeTime1-AgeTime4;
        * If is first row for that ID, set all initially to 0;
        IF FIRST.ID THEN DO;
            * Do for all 4 waves;
            DO i=1 TO 4 BY 1; 
                aDV(i)=0; aAge(i)=0; 
            END;
        END; 
        * Now copy stacked data into new variables using "wave" as index;
        aDV(wave)=DV; aAge(wave)=Age; 
        * If is last for that ID, write new line of data;
        IF LAST.ID THEN OUTPUT; 
RUN; 




